<?php

/**
 * @file
 * Contains form code for commerce_pom_order.module.
 */

/**
 * Form function to create a pom_order entity.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 * @param object $purchase_order
 *   The commerce purchase order entity.
 *
 * @return array
 *   The form array.
 */
function commerce_pom_order_entity_form(array $form, array &$form_state, $purchase_order = NULL) {
  // Show message if purchase order manager hasn't been configured yet.
  if (!_commerce_pom_configured()) {
    drupal_set_message(t('You must <a href="!url">configure</a> the Purchase Order Manager module.', [
      '!url' => url('admin/commerce/config/pom'),
    ]), 'warning');
  }

  $form['#tree'] = TRUE;

  // Determine if we're editing a purchase order.
  $edit = FALSE;
  if (!empty($purchase_order->pom_order_id)) {
    $edit = TRUE;

    // Add the line items in the PO to the form state for adding to the products
    // table.
    if (!isset($form_state['line_item_info'])) {
      $form_state += commerce_pom_order_fetch_po_line_items($purchase_order);
    }
  }

  // Set form title depending on whether we're editing or creating a PO.
  if ($edit) {
    drupal_set_title(t('Purchase Order'));

    // Add the purchase order summary text when we're editing a sent PO.
    if ($purchase_order->status != COMMERCE_POM_ORDER_STATUS_DEFAULT) {
      $po_summary_text = commerce_pom_order_get_po_summary_text($purchase_order);
    }
  }
  else {
    drupal_set_title(t('Create Purchase Order'));
  }

  // Attach our CSS and JS.
  $form['#attached']['css'][] = drupal_get_path('module', 'commerce_pom') . '/css/commerce_pom_style.css';

  $form['#attached']['library'][] = [
    'system',
    'ui.autocomplete',
  ];

  $form['#attached']['library'][] = [
    'system',
    'drupal.ajax',
  ];

  $js_settings = [
    'productAutoCompleteUrl' => url('admin/commerce/pom/product/autocomplete'),
  ];

  // Attach form.
  field_attach_form('pom_order', $purchase_order, $form, $form_state);

  // Purchase order help text.
  $form['purchase_order_description'] = [
    '#type' => 'item',
    '#title' => t('Create a Purchase Order (P.O.) whenever ordering products from suppliers/vendors. Update orders as <span class="strong">Invoice No.</span> and <span class="strong">Stock</span> are received.'),
    '#suffix' => isset($po_summary_text) ? $po_summary_text : '',
  ];

  // Check if we've got any ajax clicks.
  _commerce_pom_order_ajax_check($form, $form_state);

  // Invoice number field.
  $form['invoice_number'] = [
    '#type' => 'textfield',
    '#title' => t('Invoice Number'),
    '#description' => t('Invoice # as provided by the Vendor'),
    '#default_value' => !empty($form_state['values']['invoice_number']) ? $form_state['values']['invoice_number'] : '',
  ];

  // Activity log view.
  $form['activity_log_button'] = [
    '#type' => 'item',
    '#markup' => l(t('Activity log'), NULL, [
      'attributes' => ['class' => ['button'], 'id' => ['activity-log-button']],
      'external' => TRUE,
      'fragment' => FALSE,
    ]),
  ];

  if (!isset($form_state['activity_log_view'])) {
    $form_state['activity_log_view'] = t('<h3> Activity Log</h3>') . views_embed_view('commerce_pom_purchase_order_activity_log', 'default', $purchase_order->pom_order_id);
  }
  $form['activity_log_view'] = [
    '#type' => 'item',
    '#prefix' => '<div id="pom-activity-log-view">',
    '#suffix' => '</div>',
    '#markup' => $form_state['activity_log_view'],
  ];

  // Purchase order ID field.
  $form['purchase_order_id'] = [
    '#type' => 'textfield',
    '#title' => t('P.O. Number'),
    '#required' => TRUE,
    '#default_value' => $edit ? $purchase_order->purchase_order_id : '',
  ];

  // Set the default value for the vendor email below.
  // First, grab it from our purchase order entity if it's not a new po.
  $vendor_email_default_value = $edit ? $purchase_order->vendor_email : '';
  // If the vendor select list was changed, grab the email from the
  // email field for that vocab and set it as the default instead.
  if (isset($form_state['input']['pom_vendor'])) {
    $term = taxonomy_term_load($form_state['input']['pom_vendor'][LANGUAGE_NONE]);
    $emails = [];
    foreach ($term->pom_vendor_email[LANGUAGE_NONE] as $key => $email) {
      $emails[] = $email['email'];
    }
    $vendor_email_default_value = implode(', ', $emails);
  }
  // Vendor email.
  unset($form_state['input']['vendor_email']);
  $form['vendor_email'] = [
    '#type' => 'textfield',
    '#title' => t('Vendor Email (optional)'),
    '#description' => t('This Purchase Order will be sent to the supplier at this email address.'),
    '#default_value' => $vendor_email_default_value,
    '#prefix' => '<div id="pom-vendor-email">',
    '#suffix' => '</div>',
  ];

  // Add products table fields.
  _commerce_pom_order_add_products_table_fields($form, $form_state);

  // Add single add product fields.
  _commerce_pom_order_add_single_product_add_fields($form, $form_state);

  // Add bulk product fields.
  _commerce_pom_order_add_bulk_product_add_fields($form, $form_state);

  // Set order entity in the form state.
  $form['pom_order_entity'] = [
    '#type' => 'value',
    '#value' => $purchase_order,
  ];

  // Checkboxes.
  $form['send_email_to_vendor'] = [
    '#type' => 'checkbox',
    '#title' => t('Email copy of Purchase Order to Vendor'),
  ];

  $form['complete_with_discrepancies'] = [
    '#type' => 'checkbox',
    '#title' => t('Complete order with discrepancies'),
  ];

  // Form action buttons.
  $form['submit'] = [
    '#type' => 'submit',
    '#value' => t('Submit Purchase Order'),
    '#element_key' => 'submit-button',
    '#weight' => 100,
  ];

  $form['cancel'] = [
    '#type' => 'submit',
    '#value' => t('Cancel Purchase Order'),
    '#element_key' => 'cancel-button',
    '#weight' => 102,
  ];

  $form['save_draft'] = [
    '#type' => 'submit',
    '#value' => t('Save Draft'),
    '#element_key' => 'draft-button',
    '#weight' => 103,
  ];

  $form['delete'] = [
    '#type' => 'submit',
    '#value' => t('Delete'),
    '#submit' => ['commerce_pom_order_delete_redirect_form'],
    '#element_key' => 'delete-button',
    '#weight' => 104,
  ];

  $form['go_back'] = [
    '#type' => 'link',
    '#title' => t('Back To Main Page'),
    '#href' => 'admin/commerce/purchase-orders',
    '#element_key' => 'go-back-button',
    '#weight' => 105,
  ];

  // Add a new activity log elements.
  $form['add_a_note_link'] = [
    '#type' => 'item',
    '#markup' => l(t('+ Add a Note'), NULL, [
      'external' => TRUE,
      'fragment' => FALSE,
      'attributes' => ['id' => ['add-a-note-link']],
    ]),
  ];

  $form['new_activity_log'] = [
    '#type' => 'textarea',
    '#attributes' => ['placeholder' => t('Note about this PO')],
    '#prefix' => '<div id="pom-new-activity-log">',
    '#default_value' => '',
  ];

  $form['add_a_note_button'] = [
    '#type' => 'submit',
    '#value' => t('Add Note'),
    '#element_key' => 'add-a-note-button',
    '#validate' => array('commerce_pom_order_validate_new_activity_log'),
    '#submit' => ['commerce_pom_order_submit_new_activity_log'],
    '#suffix' => '</div>',
  ];

  // Hide some of the fields if the po is not in draft state.
  $form['pom_line_item']['#access'] = FALSE;
  if ($purchase_order->status != COMMERCE_POM_ORDER_STATUS_DEFAULT) {
    $form['pom_vendor']['#access'] = FALSE;
    $form['vendor_email']['#access'] = FALSE;
    $form['purchase_order_id']['#access'] = FALSE;
    $form['product_search']['#access'] = FALSE;
    $form['product_sku']['#access'] = FALSE;
    $form['bulk_add_products']['#access'] = FALSE;
    $form['save_draft']['#access'] = FALSE;
    $form['send_email_to_vendor']['#access'] = FALSE;

    // If the purchase order has been submitted, change the submit text.
    if ($purchase_order->status == COMMERCE_POM_ORDER_STATUS_ORDERED || $purchase_order->status == COMMERCE_POM_ORDER_STATUS_PARTIALLY_RECEIVED) {
      $form['submit']['#value'] = t('Update Purchase Order');
    }

    // PO is completed or cancelled.
    if ($purchase_order->status != COMMERCE_POM_ORDER_STATUS_ORDERED && $purchase_order->status != COMMERCE_POM_ORDER_STATUS_PARTIALLY_RECEIVED) {
      $form['invoice_number']['#access'] = FALSE;
      $form['submit']['#access'] = FALSE;
      $form['complete_with_discrepancies']['#access'] = FALSE;
      $form['cancel']['#access'] = FALSE;
    }
  }
  // Else, if it's still in draft state or is a new order.
  else {
    $form['invoice_number']['#access'] = FALSE;
    $form['cancel']['#access'] = FALSE;
    $form['complete_with_discrepancies']['#access'] = FALSE;
    $form['add_a_note_link'] = FALSE;
    $form['activity_log_button'] = FALSE;

    if (!$edit) {
      $form['delete']['#access'] = FALSE;
    }

    $form['submit']['#attributes']['class'] = ['pom-submit-purchase-order'];
  }

  // Settings for our product search autocomplete.
  $form['#after_build'][] = 'commerce_pom_order_entity_form_after_build';
  $form['#attached']['js'][] = drupal_get_path('module', 'commerce_pom_order') . '/js/commerce_pom_order.js';
  $form['#attached']['js'][] = drupal_get_path('module', 'commerce_pom_order') . '/js/keynavigator.js';
  $form['#attached']['js'][] = drupal_get_path('module', 'commerce_extra_quantity') . '/commerce_extra_quantity.js';
  $form['#attached']['css'][] = drupal_get_path('module', 'commerce_extra_quantity') . '/commerce_extra_quantity.css';
  $form['#attached']['js'][] = [
    'type' => 'setting',
    'data' => [
      'commercePomOrder' => $js_settings,
      '#attributes' => [
        'class' => ['commerce-pom-add-product-btn'],
      ],
    ],
  ];

  // Let's add a confirmation dialog on the submit button to make the user
  // confirm before sending the PO to the vendor.
  $form['#suffix'] = t('<div id="pom-dialog-message" title="Submit Purchase Order"></div>');
  if (isset($form_state['values']) || $edit) {
    drupal_add_js(['commerce_pom_order' => [
      'message' => t('<p>You\'re about to send P.O. @purchase_order_id to @vendor_name at @vendor_email.</p>', [
        '@purchase_order_id' => isset($form_state['input']['purchase_order_id']) ? $form_state['input']['purchase_order_id'] : $purchase_order->purchase_order_id,
        '@vendor_name' => isset($term) ? $term->name : $form['pom_vendor'][LANGUAGE_NONE]['#options'][$form['pom_vendor'][LANGUAGE_NONE]['#default_value'][0]],
        '@vendor_email' => $vendor_email_default_value,
      ]),
    ]], ['type' => 'setting']);
  }

  $form['#attributes'] = ['id' => ['pom-order-entity-form']];


  return $form;
}

/**
 * Ater build function for pom_order entity form.
 */
function commerce_pom_order_entity_form_after_build($form, &$form_state) {
  drupal_add_library('system', 'ui.dialog');

  return $form;
}

/**
 * Function that checks if an ajax element was clicked in the order entity form.
 */
function _commerce_pom_order_ajax_check(&$form, &$form_state) {
  if (isset($form_state['triggering_element'])) {
    switch ($form_state['triggering_element']['#element_key']) {
      case 'product-sku':
        // Add the new product to our form_state array.
        $form_state['added_products'][$form_state['values']['product_sku']] = commerce_pom_order_get_product_details($form_state['values']['product_sku']);

        // Now, fetch the on_order details.
        $on_order_result = commerce_pom_order_get_total_on_order([$form_state['added_products'][$form_state['values']['product_sku']]['product']->product_id => $form_state['added_products'][$form_state['values']['product_sku']]]);
        $quantity_received_result = commerce_pom_order_get_total_quantity_received([$form_state['added_products'][$form_state['values']['product_sku']]['product']->product_id => $form_state['added_products'][$form_state['values']['product_sku']]]);
        $form_state['added_products'][$form_state['values']['product_sku']]['on_order'] = [
          'on_order_result' => $on_order_result[$form_state['added_products'][$form_state['values']['product_sku']]['product']->product_id],
          'quantity_received_result' => $quantity_received_result[$form_state['added_products'][$form_state['values']['product_sku']]['product']->product_id],
        ];
        unset($form_state['bulk_added_products']);

        break;

      // If the remove link was clicked.
      case 'remove-link':
        unset($form_state['added_products'][$form_state['triggering_element']['#sku']]);

        break;

      // If the receive all button was clicked.
      case 'receive-all-button':
        _commerce_pom_order_populate_quantities($form, $form_state);

        break;

      // If the bulk add products search button was clicked.
      case 'bulk-add-products-search':
        $form_state['bulk_added_products'] = [];
        _commerce_pom_order_fetch_bulk_products($form, $form_state);

        break;

      // If the bulk add products add to PO button was clicked.
      case 'add-to-po-button':
        // Add the bulk added products to the added products array.
        // First remove the bulk products that have 0 quantity.
        _commerce_pom_order_remove_empty_bulk_products($form_state);
        isset($form_state['added_products']) ? $form_state['added_products'] += $form_state['bulk_added_products'] : $form_state['added_products'] = $form_state['bulk_added_products'];
        unset($form_state['bulk_added_products']);

        break;
    }
  }
}

/**
 * Form validate handler for commerce_pom_order_entity_form.
 */
function commerce_pom_order_entity_form_validate($form, &$form_state) {
  $purchase_order = $form_state['values']['pom_order_entity'];

  field_attach_form_validate('pom_order', $purchase_order, $form, $form_state);

  // If we've got open bulk add products and they've got quantities in them,
  // output an alert stating so, do it only once.
  if ($form_state['triggering_element']['#element_key'] == 'submit-button' || $form_state['triggering_element']['#element_key'] == 'draft-button') {
    if (!isset($form_state['not_empty_quantities_warning_shown']) && isset($form_state['values']['bulk_add_products']['products'])) {
      $not_empty_quantities = FALSE;
      foreach ($form_state['values']['bulk_add_products']['products'] as $sku => $product) {
        if (isset($product['order_quantity']) && $product['order_quantity'] != 0) {
          $not_empty_quantities = TRUE;
          break;
        }
      }
      if ($not_empty_quantities) {
        form_set_error('', '');
        drupal_set_message(t('There are quantities greater than 0 in the Bulk Product Add table. Are you sure you want to save this purchase order?'), 'warning');
        $form_state['not_empty_quantities_warning_shown'] = TRUE;
      }
    }
  }

  // If any of the line items have 0 for quantity display an error.
  if (!empty($form_state['values']['products'])) {
    foreach ($form_state['values']['products'] as $sku => $product) {
      if (isset($product['order_quantity']) && $product['order_quantity'] == 0) {
        form_set_error('products][' . $sku, t('Products in the purchase order cannot have 0 quantities.'));
      }
    }
  }

  // If the user clicked to cancel the order, make them confirm once.
  if ($form_state['triggering_element']['#element_key'] == 'cancel-button') {
    if (!isset($form_state['cancel_po_warning_shown'])) {
      form_set_error('', '');
      drupal_set_message(t('Are you sure you want to cancel this purchase order? This action cannot be undone.'), 'warning');
      $form_state['cancel_po_warning_shown'] = TRUE;
    }
  }

  // Fix for displaying only one message per multiple errors.
  if (isset($_SESSION['messages']['error'])) {
    $_SESSION['messages']['error'] = array_unique($_SESSION['messages']['error']);
  }
}

/**
 * Form submit handler for commerce_pom_order_entity_form.
 */
function commerce_pom_order_entity_form_submit($form, &$form_state) {
  global $user;

  // Variable that keeps track of whether this is a new order or not.
  $new_po = FALSE;

  // Save the purchase order entity.
  $purchase_order = $form_state['values']['pom_order_entity'];

  // Set the order values.
  $purchase_order->purchase_order_id = $form_state['values']['purchase_order_id'];
  // Set the purchase order's uid if it's being created at this time.
  if (empty($purchase_order->pom_order_id)) {
    $new_po = TRUE;
    $purchase_order->uid = $user->uid;
    $purchase_order->created = REQUEST_TIME;
  }
  $purchase_order->vendor_email = $form_state['values']['vendor_email'];
  $purchase_order->status = _commerce_pom_order_get_po_status($purchase_order, $form_state);
  // Set the ordered date if the purchase order is being sent.
  if ($purchase_order->status == COMMERCE_POM_ORDER_STATUS_ORDERED) {
    $purchase_order->ordered = REQUEST_TIME;
  }

  // Now, save the order.
  field_attach_submit('pom_order', $purchase_order, $form, $form_state);
  $purchase_order = commerce_pom_order_save($purchase_order);

  // Now, let's save the line items for this purchase order.
  if (!empty($form_state['values']['products'])) {
    $line_items = _commerce_pom_order_save_line_items($purchase_order, $form_state);

    // We've got to now update the purchase order pom_line_item reference field
    // with these newly inserted line items.
    $order_wrapper = entity_metadata_wrapper('pom_order', $purchase_order);
    $order_wrapper->pom_line_item->set(array_keys($line_items));
    $order_wrapper->save();
  }

  // Add the form state line item info to an array to pass to other functions.
  $line_item_data = [
    'line_item_info' => isset($form_state['line_item_info']) ? $form_state['line_item_info'] : [],
    'added_products' => isset($form_state['added_products']) ? $form_state['added_products'] : [],
  ];

  // If the email copy to vendor checkbox was checked, send an email to the
  // vendor.
  if ($form_state['values']['send_email_to_vendor'] == 1 && $purchase_order->status == COMMERCE_POM_ORDER_STATUS_ORDERED) {
    module_load_include('inc', 'commerce_pom_order', 'includes/commerce_pom_order.email');

    commerce_pom_order_send_vendor_email($purchase_order, $line_item_data);
  }

  // Log the PO activity via the message entity.
  module_load_include('inc', 'commerce_pom_order', 'includes/message/commerce_pom_order.message');
  $data = [
    'previous_po' => $form_state['build_info']['args'][0],
    'line_item_data' => $line_item_data,
    'invoice_number_added' => !empty($form_state['values']['invoice_number']) ? $form_state['values']['invoice_number'] : FALSE,
    'products' => isset($form_state['values']['products']) ? $form_state['values']['products'] : [],
  ];
  _commerce_pom_order_create_po_activity_log($purchase_order, $data, $new_po);

  // Output success message and redirect the form.
  if ($new_po) {
    $message = t('Purchase Order @purchase_order_id has been successfully created.', [
      '@purchase_order_id' => $purchase_order->purchase_order_id,
    ]);
  }
  else {
    $message = t('Purchase Order @purchase_order_id has been successfully updated.', [
      '@purchase_order_id' => $purchase_order->purchase_order_id,
    ]);
  }
  drupal_set_message($message);

  $form_state['redirect'] = 'admin/commerce/purchase-orders';
}

/**
 * Function that processes and saves the line items in a PO.
 *
 * @param object $purchase_order
 *   The purchase order entity.
 * @param array $form_state
 *   The form state array.
 *
 * @return array
 *   The saved line items keyed by the pom_line_item_id.
 */
function _commerce_pom_order_save_line_items($purchase_order, array &$form_state) {
  // First, delete the line items that were in the purchase order that have now
  // been removed from the form_state.
  $line_item_ids = [];
  foreach ($purchase_order->pom_line_item[LANGUAGE_NONE] as $key => $value) {
    $line_item_exits = FALSE;
    foreach ($form_state['values']['products'] as $sku => $line_item_info) {
      if ($value['target_id'] == $line_item_info['line_item_id']) {
        $line_item_exits = TRUE;
        break;
      }
    }
    if (!$line_item_exits) {
      $line_item_ids[] = $value['target_id'];
    }
  }
  commerce_pom_line_item_delete_multiple($line_item_ids);

  // Now, save the new values.
  $line_items = commerce_pom_line_item_save_line_items($purchase_order, $form_state);

  return $line_items;
}

/**
 * Form submit handler for the purchase order delete button.
 */
function commerce_pom_order_delete_redirect_form($form, &$form_state) {
  // Redirect to our commerce purchase order confirm delete form.
  $form_state['redirect'] = 'admin/commerce/purchase-orders/' . $form_state['values']['pom_order_entity']->pom_order_id . '/delete';
}

/**
 * Form callback: confirmation form for deleting a commerce purchase order.
 *
 * @see confirm_form()
 */
function commerce_pom_order_delete_form($form, &$form_state, $purchase_order) {
  $form_state['pom_order'] = $purchase_order;

  // Ensure this include file is loaded when the form is rebuilt from the cache.
  $form_state['build_info']['files']['form'] = drupal_get_path('module', 'commerce_pom_order') . '/includes/commerce_pom_order.forms.inc';

  $form['#submit'][] = 'commerce_pom_order_delete_form_submit';

  $form = confirm_form($form,
    t('Are you sure you want to delete Purchase Order ID %purchase_order_id?', ['%purchase_order_id' => $purchase_order->purchase_order_id]),
    'admin/commerce/purchase-orders',
    t('Deleting this purchase order cannot be undone.'),
    t('Delete'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Submit callback for commerce_pom_order_delete_form().
 */
function commerce_pom_order_delete_form_submit($form, &$form_state) {
  $purchase_order = $form_state['pom_order'];

  if (commerce_pom_order_delete($purchase_order->pom_order_id)) {
    drupal_set_message(t('Purchase Order %purchase_order_id has been deleted.', ['%purchase_order_id' => $purchase_order->purchase_order_id]));
    watchdog('commerce_pom_order', 'Deleted purchase order %id (Purchase Order ID: @purchase_order_id).', [
      '%id' => $purchase_order->pom_order_id,
      '@purchase_order_id' => $purchase_order->purchase_order_id,
    ], WATCHDOG_NOTICE);
  }
  else {
    drupal_set_message(t('Purchase Order ID %purchase_order_id could not be deleted.', ['%purchase_order_id' => $purchase_order->purchase_order_id]), 'error');
  }
  $form_state['redirect'] = 'admin/commerce/purchase-orders';
}

/**
 * Ajax callback for the pom_vendor field in commerce_pom_order_entity_form.
 */
function commerce_pom_order_vendor_ajax_callback($form, &$form_state) {
  return $form['vendor_email'];
}

/**
 * Validation handler for the product search input.
 */
function _commerce_pom_order_validate_product_input($element, &$form_state, $form) {
  // Always set this input back to empty.
  unset($form_state['input']['input']);
  form_set_value($element, '', $form_state);
}

/**
 * Returns the status of a purchase order depending on the form_state.
 *
 * @param object $purchase_order
 *   The purchase order entity.
 * @param array $form_state
 *   The form state array.
 *
 * @return int
 *   The status of the purchase order.
 */
function _commerce_pom_order_get_po_status($purchase_order, array $form_state) {
  // PO is in draft state by default.
  $status = COMMERCE_POM_ORDER_STATUS_DEFAULT;

  if ($form_state['triggering_element']['#element_key'] == 'submit-button') {
    // PO is in ordered state.
    $status = COMMERCE_POM_ORDER_STATUS_ORDERED;

    // If completed with discrepancies checkbox is checked, set it to that.
    if ($form_state['values']['complete_with_discrepancies'] == 1) {
      $status = COMMERCE_POM_ORDER_STATUS_COMPLETED_WD;
    }
    else {
      // Check if quantity was received for any of the line items.
      $quantity_received = FALSE;
      if (!empty($form_state['values']['products'])) {
        foreach ($form_state['values']['products'] as $sku => $product) {
          if ((isset($product['quantity_received']) && !empty($product['quantity_received'])) || isset($product['quantity_received_summary'])) {
            // If quantities have been received, check if we have fulfillment.
            $quantity_received = TRUE;
            $quantity_fulfilled[$sku] = commerce_pom_order_line_item_quantity_fulfilled($product);
          }
        }
      }

      // PO is in partially received state.
      if ($quantity_received) {
        $status = COMMERCE_POM_ORDER_STATUS_PARTIALLY_RECEIVED;
      }

      // If all items have been fulfilled, set it to completed state.
      if (isset($quantity_fulfilled) && !in_array(FALSE, $quantity_fulfilled)) {
        // PO is in fulfilled state.
        $status = COMMERCE_POM_ORDER_STATUS_COMPLETED;
      }
    }
  }
  // If cancel PO was clicked, set it to that.
  elseif ($form_state['triggering_element']['#element_key'] == 'cancel-button') {
    $status = COMMERCE_POM_ORDER_STATUS_CANCELLED;
  }

  // Final call to alter the status.
  drupal_alter('commerce_pom_order_status', $purchase_order, $form_state, $status);

  return $status;
}

/**
 * Function which will populate the quantity received field for all line items.
 *
 * The populated quantity will be calculated as quantity ordered - received.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 */
function _commerce_pom_order_populate_quantities(array $form, array &$form_state) {
  $line_item_ids = [];
  $product_ids = [];

  foreach ($form_state['added_products'] as $sku => $product) {
    $line_item_ids[] = $form_state['line_item_info'][$sku]['line_item']->pom_line_item_id;
    $product_ids[$product['product']->product_id] = $product['product'];
  }
  // Grab the total received for each line item.
  $total_received = commerce_pom_order_get_total_quantity_received(
    $product_ids,
    $line_item_ids
  );

  // Now, set it in the form_state array.
  foreach ($form_state['added_products'] as $sku => $product) {
    if (isset($total_received[$product['product']->product_id])) {
      $quantity_received = (int) $form_state['line_item_info'][$sku]['line_item']->quantity_ordered - $total_received[$product['product']->product_id][$form_state['line_item_info'][$sku]['line_item']->pom_line_item_id]->total_quantity_received;
    }
    else {
      $quantity_received = (int) $form_state['line_item_info'][$sku]['line_item']->quantity_ordered;
    }
    $form_state['added_products'][$sku]['quantity_received'] = $quantity_received > 0 ? $quantity_received : 0;
  }
}

/**
 * Add the line items to the products table form element.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 */
function _commerce_pom_order_add_products_table_fields(array &$form, array &$form_state) {
  $purchase_order = $form['#entity'];
  $edit = !empty($purchase_order->pom_order_id) ? TRUE : FALSE;
  $bulk_add = isset($form_state['bulk_added_products']) ? TRUE : FALSE;
  $elements = [];

  // Header for the products table.
  $bulk_add_header = $header = [
    'product' => t('Product'),
    'sku' => t('Sku'),
    'in_stock' => t('In Stock'),
    'on_order' => t('On Order'),
    'order_quantity' => t('Order Qty'),
  ];

  // The form element for the main products table is $form['products'].
  $elements[] = 'products';

  // If we're also adding bulk products, the form element is a bit different.
  if ($bulk_add) {
    $elements[] = 'bulk_add_products';
  }

  // Display a table of all the products added to this purchase order.
  // First go through the added products and generate form elements for each.
  foreach ($elements as $element_type) {
    // If we're adding the products to the main product table.
    if ($element_type == 'products') {
      // Our main products table.
      $form['products'] = [
        '#type' => 'fieldset',
        '#title' => !$edit ? t('Order: P.O. Products') : '',
        '#prefix' => '<div id="pom-products-table">',
        '#suffix' => '</div>',
        '#weight' => 10,
      ];

      // Keeps track of how many times shipment for a single PO was taken.
      $quantity_received_times = [];

      // The products that go in the main table are added to
      // $form_state['added_products'] array.
      $added_products = isset($form_state['added_products']) ? $form_state['added_products'] : [];

      $element = &$form['products'];
    }
    // Else, if we're adding bulk products to the bulk products table.
    else {
      $form['bulk_add_products']['products'] = [
        '#type' => 'fieldset',
        '#title' => '',
        '#prefix' => '<div id="pom-bulk-products-table">',
        '#suffix' => '</div>',
        '#weight' => 10,
      ];

      // The bulk products.
      $added_products = $form_state['bulk_added_products'];

      $element = &$form['bulk_add_products']['products'];
    }

    // Now add the products to their respective tables.
    if (!empty($added_products)) {
      foreach ($added_products as $sku => $product) {
        // Save the line item id as a hidden value.
        $element[$sku]['line_item_id'] = [
          '#type' => 'value',
          '#value' => isset($form_state['line_item_info'][$sku]['line_item']->pom_line_item_id) ? $form_state['line_item_info'][$sku]['line_item']->pom_line_item_id : '',
          '#hidden' => TRUE,
        ];

        // Save the product category as a hidden value.
        $element[$sku]['product_category'] = [
          '#type' => 'value',
          '#value' => $product['product_category'],
          '#hidden' => TRUE,
        ];

        // Save the product brand as a hidden value.
        $element[$sku]['product_brand'] = [
          '#type' => 'value',
          '#value' => $product['product_brand'],
          '#hidden' => TRUE,
        ];

        // Add the title field.
        $element[$sku]['title'] = [
          '#type' => 'item',
          '#markup' => l($product['title'], $product['view_url'], [
            'attributes' => [
              'target' => '_blank',
            ],
          ]),
        ];

        // Add the sku field.
        $element[$sku]['sku'] = [
          '#type' => 'value',
          '#value' => $product['sku'],
        ];

        // Add the in_stock & on_order fields if we're in new/draft status.
        if ($purchase_order->status == COMMERCE_POM_ORDER_STATUS_DEFAULT) {
          // Add the in_stock field.
          $element[$sku]['in_stock'] = [
            '#type' => 'value',
            '#value' => $product['in_stock'],
          ];

          // Add the on_order field which is the total qty ordered - received.
          $on_order = !empty($product['on_order']['on_order_result']) ? $product['on_order']['on_order_result']->total_quantity_ordered : 0;
          $quantity_received = !empty($product['on_order']['quantity_received_result']) ? $product['on_order']['quantity_received_result']->total_quantity_received : 0;
          $element[$sku]['on_order'] = [
            '#type' => 'value',
            '#value' => (int) $on_order - $quantity_received,
          ];
        }
        // If we're not showing the in_stock & on_order fields, remove them
        // from the table headers.
        else {
          unset($header['in_stock']);
          unset($header['on_order']);
        }

        // When adding an already submitted PO.
        if (!$bulk_add && $purchase_order->status != COMMERCE_POM_ORDER_STATUS_DEFAULT) {
          // Add the order_quantity as a field showing how many was ordered.
          $element[$sku]['order_quantity'] = [
            '#type' => 'value',
            '#value' => (int) $form_state['line_item_info'][$sku]['line_item']->quantity_ordered,
          ];
          // Change the order_quantity header title.
          $header['order_quantity'] = t('QTY Ordered');

          // Display the quantity received fields if we've got active PO's.
          if ($purchase_order->status != COMMERCE_POM_ORDER_STATUS_DEFAULT) {
            // The summary of all qty received entries for this PO line item.
            if (!empty($form_state['line_item_info'][$sku]['quantity_received_items'])) {
              foreach ($form_state['line_item_info'][$sku]['quantity_received_items'] as $key => $quantity_received) {
                // Add a column for each quantity received.
                $element[$sku]['quantity_received_summary'][$quantity_received->received] = [
                  '#type' => 'value',
                  '#value' => (int) $quantity_received->quantity,
                ];

                // Add an entry for every time a shipment was taken for this PO.
                $quantity_received_times[$quantity_received->received] = t('Received: @date, Invoice #: @invoice_number', [
                  '@date' => date('m/d/y', $quantity_received->received),
                  '@invoice_number' => $quantity_received->invoice_number,
                ]);
              }
            }

            // The quantity_received text field for active PO's only.
            if ($purchase_order->status == COMMERCE_POM_ORDER_STATUS_ORDERED || $purchase_order->status == COMMERCE_POM_ORDER_STATUS_PARTIALLY_RECEIVED) {
              unset($form_state['input']['products'][$sku]['quantity_received']);
              $element[$sku]['quantity_received'] = [
                '#type' => 'textfield',
                '#size' => 4,
                '#maxlength' => 10,
                '#element_validate' => ['element_validate_integer_positive'],
                '#default_value' => (isset($product['quantity_received']) ? $product['quantity_received'] : (isset($form_state['values']['products'][$sku]['quantity_received']) ? $form_state['values']['products'][$sku]['quantity_received'] : '')),
              ];
            }
          }
        }
        // Else, if this is a bulk add and the PO hasn't been submitted yet.
        else {
          // Add the order_quantity field.
          // First, determine the quantity ordered default value.
          if (isset($form_state['line_item_info'][$sku])) {
            $quantity_ordered_default_value = (int) $form_state['line_item_info'][$sku]['line_item']->quantity_ordered;
          }
          elseif (isset($form_state['values']['products'][$sku]['order_quantity'])) {
            $quantity_ordered_default_value = $form_state['values']['products'][$sku]['order_quantity'];
          }
          elseif (isset($form_state['input']['bulk_add_products']['products'][$sku]['order_quantity'])) {
            $quantity_ordered_default_value = $form_state['input']['bulk_add_products']['products'][$sku]['order_quantity'];
          }
          else {
            $quantity_ordered_default_value = 0;
          }

          $element[$sku]['order_quantity'] = [
            '#type' => 'textfield',
            '#size' => 4,
            '#maxlength' => 10,
            '#required' => TRUE,
            '#element_validate' => ['element_validate_integer'],
            '#default_value' => $quantity_ordered_default_value,
          ];

          // Now, determine if lot quantity is enabled for this product.
          if (isset($product['product']->pom_lot_quantity) && $product['product']->pom_lot_quantity[LANGUAGE_NONE][0]['value'] != 1) {
            // Enable the increment/decrement buttons.
            $element[$sku]['order_quantity']['#theme_wrappers'] = [
              'quantity_plusminus_wrapper',
              'form_element',
            ];

            // Make the quantity step by the lot quantity amount.
            $element[$sku]['order_quantity']['#limit_down'] = 0;
            $element[$sku]['order_quantity']['#amount'] = (int) $product['product']->pom_lot_quantity[LANGUAGE_NONE][0]['value'];
            // Set the default value to the lot quantity as well.
            if (isset($form_state['line_item_info'][$sku])) {
              $quantity_ordered_default_value = (int) $form_state['line_item_info'][$sku]['line_item']->quantity_ordered;
            }
            elseif (isset($form_state['values']['products'][$sku]['order_quantity'])) {
              $quantity_ordered_default_value = $form_state['values']['products'][$sku]['order_quantity'];
            }
            elseif (isset($form_state['input']['bulk_add_products']['products'][$sku]['order_quantity'])) {
              $quantity_ordered_default_value = $form_state['input']['bulk_add_products']['products'][$sku]['order_quantity'];
            }
            else {
              $quantity_ordered_default_value = 0;
            }

            $element[$sku]['order_quantity']['#default_value'] = $quantity_ordered_default_value;
            // Also, don't allow the user to edit the field manually.
            $element[$sku]['order_quantity']['#attributes'] = ['readonly' => 'readonly'];
          }
        }

        // If we're not bulk adding, show the operations fields.
        // Form the operation links.
        if ($element_type == 'products' && $purchase_order->status == COMMERCE_POM_ORDER_STATUS_DEFAULT) {
          $element[$sku]['operations'] = [
            '#type' => 'button',
            '#prefix' => $product['view_link'],
            '#value' => t('remove'),
            '#ajax' => [
              'callback' => 'commerce_pom_order_bulk_add_to_po_ajax_callback',
              'wrapper' => 'pom-products-table',
            ],
            '#limit_validation_errors' => [],
            '#element_key' => 'remove-link',
            '#name' => 'remove-link-' . $sku,
            '#sku' => $sku,
          ];

          $header += ['operation' => t('Operation')];
        }
      }

      // As some line items might not have been received when others were for a
      // shipment, we need to make sure we input 0's in the quantity received
      // summary fields for those not received as it will mess up the table.
      if (!$bulk_add && $purchase_order->status != COMMERCE_POM_ORDER_STATUS_DEFAULT) {
        $refactored_table = _commerce_pom_order_refactor_po_quantity_received_summary($purchase_order, $quantity_received_times, $header, $element);
        $element = $refactored_table['rows'];
        $header = $refactored_table['header'];
      }

      // Add the receive all button if we're not bulk adding products and the
      // status is sent.
      if (!$bulk_add && ($purchase_order->status == COMMERCE_POM_ORDER_STATUS_ORDERED || $purchase_order->status == COMMERCE_POM_ORDER_STATUS_PARTIALLY_RECEIVED)) {
        // Add the qty received input header as the last column.
        $header += ['quantity_received' => t('QTY Received')];

        $element['receive_all'] = [
          '#type' => 'button',
          '#value' => t('Receive All'),
          '#ajax' => [
            'callback' => 'commerce_pom_order_bulk_add_to_po_ajax_callback',
            'wrapper' => 'pom-products-table',
          ],
          '#limit_validation_errors' => [],
          '#element_key' => 'receive-all-button',
          '#prefix' => '<div id="pom-receive-all-button">',
          '#suffix' => '</div>',
        ];
      }
      // Else, if we're adding bulk products, add the "Add to PO" button.
      elseif ($bulk_add) {
        $element['add_to_po'] = [
          '#type' => 'button',
          '#value' => t('Add to P.O.'),
          '#ajax' => [
            'callback' => 'commerce_pom_order_bulk_add_to_po_ajax_callback',
          ],
          '#limit_validation_errors' => [],
          '#element_key' => 'add-to-po-button',
        ];
      }
    }
    // Theme the products fieldset elements.
    // First, make sure we've got all headers.
    // If we're adding to the bulk product table, denote that for the theme.
    if ($element_type == 'products') {
      $element['#header'] = $header;
    }
    else {
      $element['#header'] = $bulk_add_header;
      $element['#bulk_add'] = TRUE;
    }

    $element['#theme'] = 'commerce_pom_order_products_form_table';
  }
}

/**
 * Add the single product add ajax fields to the form.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 */
function _commerce_pom_order_add_single_product_add_fields(array &$form, array &$form_state) {
  // The product autocomplete search field.
  $form['product_search'] = [
    '#type' => 'textfield',
    '#title' => t('<h3>Add Product</h3>'),
    '#description' => t('Search for Products to add to your P.O.'),
    '#size' => 60,
    '#attributes' => [
      'class' => [
        'commerce-pom-product-autocomplete',
        'commerce-pom-product-search',
      ],
      'placeholder' => t('SKU or Product Keywords'),
    ],
    '#element_validate' => ['_commerce_pom_order_validate_product_input'],
    '#weight' => 11,
  ];

  // A hidden field which will grab the product sku added in the product search.
  $form['product_sku'] = [
    '#type' => 'textfield',
    '#title' => NULL,
    '#ajax' => [
      'callback' => 'commerce_pom_order_bulk_add_to_po_ajax_callback',
      'wrapper' => 'pom-products-table',
    ],
    '#attributes' => [
      'class' => ['commerce-pom-product-sku-input'],
    ],
    '#element_key' => 'product-sku',
    '#prefix' => '<div class="element-invisible">',
    '#suffix' => '</div>',
    '#weight' => 11,
  ];
}

/**
 * Add the bulk product add ajax fields to the form.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form_state array.
 */
function _commerce_pom_order_add_bulk_product_add_fields(array &$form, array &$form_state) {
  // Only display the bulk add products if we have brand and category vocabs
  // configured.
  if (!empty(variable_get('commerce_pom_brand_field', '')) && !empty(variable_get('commerce_pom_category_field', ''))) {
    $form['bulk_add_products'] = [
      '#type' => 'fieldset',
      '#prefix' => t('<div id="pom-bulk-add-products-table"><h3>Bulk Add Products</h3>'),
      '#suffix' => '</div>',
      '#title' => t('Find Products'),
      '#weight' => 11,
    ];

    // Let's load all the terms from the brand vocabulary.
    $field = field_info_field(variable_get('commerce_pom_brand_field', ''));
    $vid = db_query('SELECT vid FROM {taxonomy_vocabulary} WHERE machine_name = :machine_name', [
      ':machine_name' => $field['settings']['allowed_values'][0]['vocabulary'],
    ])->fetchField();
    $terms = db_query('SELECT tid, name FROM {taxonomy_term_data} WHERE vid = :vid ORDER BY name', [
      ':vid' => $vid,
    ]);
    $options = ['' => t('Any')];
    foreach ($terms as $term) {
      $options[$term->tid] = $term->name;
    }

    $form['bulk_add_products']['search_fields'] = [
      '#type' => 'fieldset',
    ];

    // Add the brand field.
    $form['bulk_add_products']['search_fields']['brand'] = [
      '#type' => 'select',
      '#title' => t('Brand'),
      '#options' => $options,
      '#multiple' => TRUE,
      '#size' => 8,
    ];

    // Now, get the terms from the category vocabulary.
    $field = field_info_field(variable_get('commerce_pom_category_field', ''));
    $vid = db_query('SELECT vid FROM {taxonomy_vocabulary} WHERE machine_name = :machine_name', [
      ':machine_name' => $field['settings']['allowed_values'][0]['vocabulary'],
    ])->fetchField();
    $terms = db_query('SELECT tid, name FROM {taxonomy_term_data} WHERE vid = :vid ORDER BY name', [
      ':vid' => $vid,
    ]);
    $options = ['' => t('Any')];
    foreach ($terms as $term) {
      $options[$term->tid] = $term->name;
    }

    // Add the category field.
    $form['bulk_add_products']['search_fields']['category'] = [
      '#type' => 'select',
      '#title' => t('Product Category'),
      '#options' => $options,
      '#multiple' => TRUE,
      '#size' => 8,
    ];

    // Search button.
    $form['bulk_add_products']['search_fields']['submit'] = [
      '#type' => 'submit',
      '#value' => t('Search'),
      '#ajax' => [
        'callback' => 'commerce_pom_order_bulk_search_ajax_callback',
        'wrapper' => 'pom-bulk-add-products-table',
      ],
      '#limit_validation_errors' => [],
      '#element_key' => 'bulk-add-products-search',
      '#executes_submit_callback' => FALSE,
    ];

    // Add the bulk products table by re-using the products table function.
    _commerce_pom_order_add_products_table_fields($form, $form_state);
  }
}

/**
 * Ajax callback for the bulk add products search button.
 */
function commerce_pom_order_bulk_search_ajax_callback($form, &$form_state) {
  return $form['bulk_add_products'];
}

/**
 * Query and fetch all products that reference the brand and category.
 *
 * @param array $form
 *   The form array.
 * @param array $form_state
 *   The form state array.
 */
function _commerce_pom_order_fetch_bulk_products(array &$form, array &$form_state) {
  module_load_include('inc', 'commerce_pom_order', 'includes/commerce_pom_order.common');

  $brand_tid = array_filter($form_state['input']['bulk_add_products']['search_fields']['brand']);
  $category_tid = array_filter($form_state['input']['bulk_add_products']['search_fields']['category']);

  // If both brand and category fields are empty, don't go further.
  if (empty($brand_tid) && empty($category_tid)) {
    drupal_set_message(t('Both brand and category fields cannot be empty.'), 'error');
    return;
  }

  $nodes = commerce_pom_order_get_referenced_nodes(variable_get('commerce_pom_product_display_bundles'), $brand_tid, $category_tid);

  if (!empty($nodes)) {
    $fields = commerce_pom_order_get_product_reference_fields(variable_get('commerce_pom_product_display_bundles'));
    if (!empty($fields)) {
      $products = commerce_pom_order_get_node_products($nodes, $fields);

      if (!empty($products)) {
        // Fetch the product details for each and add the products to the
        // form_state so we can add to our product table.
        $product_details = commerce_pom_order_get_product_details($products);

        // Now, fetch the on_order details.
        $on_order_result = commerce_pom_order_get_total_on_order($products);
        $quantity_received_result = commerce_pom_order_get_total_quantity_received($products);

        // Go through all the products and add their info to the form_state.
        foreach ($product_details as $sku => $result) {
          $result['on_order'] = [
            'on_order_result' => isset($on_order_result[$result['product']->product_id]) ? $on_order_result[$result['product']->product_id] : [],
            'quantity_received_result' => isset($quantity_received_result[$result['product']->product_id]) ? $quantity_received_result[$result['product']->product_id] : [],
          ];
          $form_state['bulk_added_products'][$sku] = $result;
        }
      }
    }
  }
}

/**
 * Ajax callback for adding the bulk products to the products table.
 */
function commerce_pom_order_bulk_add_to_po_ajax_callback($form, &$form_state) {
  $commands = [];
  $commands[] = ajax_command_replace('#pom-products-table', drupal_render($form['products']));
  $commands[] = ajax_command_replace('#pom-bulk-products-table', '');

  return ['#type' => 'ajax', '#commands' => $commands];
}

/**
 * Omit products from the bulk_added_products list if they have 0 for quantity.
 */
function _commerce_pom_order_remove_empty_bulk_products(&$form_state) {
  foreach ($form_state['input']['bulk_add_products']['products'] as $sku => $product) {
    if ($product['order_quantity'] == 0) {
      unset($form_state['bulk_added_products'][$sku]);
    }
  }
}

/**
 * Validate function for adding a new manual activity log to a purchase order.
 */
function commerce_pom_order_validate_new_activity_log($form, &$form_state) {
  if (empty($form_state['values']['new_activity_log'])) {
    form_set_error('new_activity_log', t('Activity log cannot be empty.'));
  }
}

/**
 * Submit function that adds a new manual activity log to a purchase order.
 */
function commerce_pom_order_submit_new_activity_log($form, &$form_state) {
  $purchase_order = $form_state['values']['pom_order_entity'];

  $message = message_create('commerce_pom_purchase_order_activity', array('uid' => $purchase_order->uid));

  // Wrap message in div.
  $message_to_insert = '<div class="pom-log-entry pom-log-manual">' . $form_state['values']['new_activity_log'] . '</div>';

  $wrapper = entity_metadata_wrapper('message', $message);
  $wrapper->pom_purchase_order->set($purchase_order);
  $wrapper->pom_activity_summary->set(array('value' => $message_to_insert, 'format' => 'full_html'));
  $wrapper->save();

  drupal_set_message(t('The note has been successfully added to this purchase order\'s activity log.'));
}
